=== modified file '__init__.py'
--- __init__.py	2009-04-10 20:07:41 +0000
+++ __init__.py	2009-05-27 18:57:14 +0000
@@ -77,6 +77,7 @@
     import tests.is_clean
     import tests.test_cbranch
     import tests.test_conflict_diff
+    from bzrlib.plugins.bzrtools.tests import test_fetch_ghosts
     import tests.test_link_tree
     import tests.test_patch
     import tests.test_rspush
@@ -93,6 +94,7 @@
     result.addTest(zap.test_suite())
     result.addTest(TestLoader().loadTestsFromModule(tests.test_dotgraph))
     result.addTest(TestLoader().loadTestsFromModule(tests.is_clean))
+    result.addTest(TestLoader().loadTestsFromModule(test_fetch_ghosts))
     result.addTest(TestLoader().loadTestsFromModule(tests.test_link_tree))
     result.addTest(TestLoader().loadTestsFromModule(tests.test_patch))
     result.addTest(TestLoader().loadTestsFromModule(tests.test_rspush))

=== modified file 'command_classes.py'
--- command_classes.py	2009-04-10 20:07:41 +0000
+++ command_classes.py	2009-05-27 18:57:14 +0000
@@ -120,7 +120,7 @@
     takes_options = [Option('no-fix', help="Skip additional synchonization.")]
     def run(self, branch=None, no_fix=False):
         from fetch_ghosts import fetch_ghosts
-        fetch_ghosts(branch, no_fix)
+        fetch_ghosts(branch, do_reconcile=not no_fix)
 
 strip_help="""Strip the smallest prefix containing num leading slashes  from \
 each file name found in the patch file."""

=== modified file 'fetch_ghosts.py'
--- fetch_ghosts.py	2007-04-04 18:12:00 +0000
+++ fetch_ghosts.py	2009-05-27 18:57:14 +0000
@@ -14,73 +14,92 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 from bzrlib.branch import Branch
-from bzrlib.errors import NoSuchRevision, InstallFailed, BzrCommandError
-def fetch_ghosts(branch, no_fix):
-    """Install ghosts from copies in another branch."""
-    this_branch = Branch.open_containing('.')[0]
-    if branch is None:
-        branch = this_branch.get_parent()
-        if branch is None:
-            raise BzrCommandError('No branch specified and no location saved.')
-        else:
-            print "Using saved location %s" % branch
-
-    other_branch = Branch.open_containing(branch)[0]
-    installed = []
-    failed = []
-    lock_other = this_branch.base != other_branch.base
-
-    this_branch.lock_write()
-    try:
-        if lock_other:
-            other_branch.lock_read()
+from bzrlib.trace import note
+from bzrlib.errors import NoSuchRevision, BzrCommandError
+
+class GhostFetcher(object):
+
+    @classmethod
+    def from_cmdline(klass, other):
+        this_branch = Branch.open_containing('.')[0]
+        if other is None:
+            other = this_branch.get_parent()
+            if other is None:
+                raise BzrCommandError('No branch specified and no location'
+                                      ' saved.')
+            else:
+                note("Using saved location %s.", branch)
+        other_branch = Branch.open_containing(other)[0]
+        return klass(this_branch, other_branch)
+
+    def __init__(self, this_branch, other_branch):
+        self.this_branch = this_branch
+        self.other_branch = other_branch
+
+    def run(self):
+        lock_other = self.this_branch.base != self.other_branch.base
+        self.this_branch.lock_write()
         try:
-            # Because iter_ghosts tests for existence after our last fetch
-            # is complete, it won't falsely report an ancestor as a ghost.
-            # Yay iterators!
-            ghosts = iter_ghosts(this_branch)
-            for revision in ghosts:
+            if lock_other:
+                self.other_branch.lock_read()
+            try:
+                return self._run_locked()
+            finally:
+                if lock_other:
+                    self.other_branch.unlock()
+        finally:
+            self.this_branch.unlock()
+
+    def iter_ghosts(self):
+        """Find all ancestors that aren't stored in this branch."""
+        seen = set()
+        lines = [self.this_branch.last_revision()]
+        if lines[0] is None:
+            return
+        while len(lines) > 0:
+            new_lines = []
+            for line in lines:
+                if line in seen:
+                    continue
+                seen.add(line)
                 try:
-                    this_branch.fetch(other_branch, revision)
-                    installed.append(revision)
-                except InstallFailed:
-                    failed.append(revision)
-            else:
-                if this_branch.last_revision() is None:
-                    print "No revisions in branch."
-            if len(installed) > 0:
-                print "Installed:"
-                for rev in installed:
-                    print rev
-            if len(failed) > 0:
-                print "Still missing:"
-                for rev in failed:
-                    print rev
-        finally:
-            if lock_other:
-                other_branch.unlock()
-    finally:
-        this_branch.unlock()
-
-    if not no_fix and len(installed) > 0:
+                    revision = self.this_branch.repository.get_revision(line)
+                    new_lines.extend(revision.parent_ids)
+                except NoSuchRevision:
+                    yield line
+            lines = new_lines
+
+    def _run_locked(self):
+        installed = []
+        failed = []
+        if self.this_branch.last_revision() is None:
+            print "No revisions in branch."
+            return
+        # Because iter_ghosts tests for existence after our last fetch
+        # is complete, it won't falsely report an ancestor as a ghost.
+        # Yay iterators!
+        ghosts = self.iter_ghosts()
+        for revision in ghosts:
+            try:
+                self.this_branch.fetch(self.other_branch, revision)
+                installed.append(revision)
+            except NoSuchRevision:
+                failed.append(revision)
+        return installed, failed
+
+def fetch_ghosts(branch, do_reconcile):
+    """Install ghosts from copies in another branch."""
+    installed, failed = GhostFetcher.from_cmdline(branch).run()
+    if len(installed) > 0:
+        print "Installed:"
+        for rev in installed:
+            print rev
+    if len(failed) > 0:
+        print "Still missing:"
+        for rev in failed:
+            print rev
+    if do_reconcile and len(installed) > 0:
         from bzrlib.builtins import cmd_reconcile
         cmd_reconcile().run(".")
 
-def iter_ghosts(branch):
-    """Find all ancestors that aren't stored in this branch."""
-    seen = set()
-    lines = [branch.last_revision()]
-    if lines[0] is None:
-        return
-    while len(lines) > 0:
-        new_lines = []
-        for line in lines:
-            if line in seen:
-                continue
-            seen.add(line)
-            try:
-                revision = branch.repository.get_revision(line)
-                new_lines.extend(revision.parent_ids)
-            except NoSuchRevision:
-                yield line
-        lines = new_lines
+

